@@ -4,6 +4,7 @@ use crate::config::ConfigRc;
 use crate::crypto::{self, SigningAlgorithm};
 use crate::email_address::EmailAddress;
 use crate::error::{BrokerError, BrokerResult};
+use crate::metrics;
 use crate::router::router;
 use crate::utils::{http::ResponseExt, real_ip, BoxError, BoxFuture};
 use bytes::{Bytes, BytesMut};
@@ -39,6 +40,27 @@ pub struct Session {
     pub bridge_data: BridgeData,
 }
 
+/// Response types we support.
+#[derive(Clone, Copy, Serialize, Deserialize)]
+pub enum ResponseType {
+    #[serde(rename = "id_token")]
+    IdToken,
+    // NOTE: This type is outside the Portier spec, but we support it in this implementation for
+    // compatibility with other OpenID Connect clients.
+    #[serde(rename = "code")]
+    Code,
+}
+
+impl ResponseType {
+    /// Get the default response mode for this response type.
+    pub fn default_response_mode(self) -> ResponseMode {
+        match self {
+            ResponseType::IdToken => ResponseMode::Fragment,
+            ResponseType::Code => ResponseMode::Query,
+        }
+    }
+}
+
 /// Response modes we support.
 #[derive(Clone, Copy, Serialize, Deserialize)]
 pub enum ResponseMode {
@@ -46,6 +68,21 @@ pub enum ResponseMode {
     Fragment,
     #[serde(rename = "form_post")]
     FormPost,
+    // NOTE: This mode is outside the Portier spec, but we support it in this implementation for
+    // compatibility with other OpenID Connect clients.
+    #[serde(rename = "query")]
+    Query,
+}
+
+impl ResponseMode {
+    /// Return the querystring value for this response mode.
+    pub fn as_str(&self) -> &str {
+        match self {
+            ResponseMode::Fragment => "fragment",
+            ResponseMode::FormPost => "form_post",
+            ResponseMode::Query => "query",
+        }
+    }
 }
 
 /// Parameters used to return to the relying party
@@ -64,6 +101,7 @@ pub struct SessionData {
     pub return_params: ReturnParams,
     pub email: String,
     pub email_addr: EmailAddress,
+    pub response_type: ResponseType,
     pub nonce: String,
     pub signing_alg: SigningAlgorithm,
 }
@@ -90,6 +128,8 @@ pub struct Context {
     pub catalog_idx: usize,
     /// Parameters used to return to the relying party
     pub return_params: Option<ReturnParams>,
+    /// Whether the response should contain a JSON body
+    pub want_json: bool,
 }
 
 impl Context {
@@ -110,19 +150,13 @@ impl Context {
         parse_form_encoded(&self.body)
     }
 
-    /// Whether this request wants a JSON response.
-    pub fn want_json(&self) -> bool {
-        self.headers
-            .get(hyper::header::ACCEPT)
-            .map_or(false, |accept| accept == "application/json")
-    }
-
     /// Start a session by filling out the common part.
     pub async fn start_session(
         &mut self,
         client_id: &str,
         email: &str,
         email_addr: &EmailAddress,
+        response_type: ResponseType,
         nonce: &str,
         signing_alg: SigningAlgorithm,
         ip: IpAddr,
@@ -139,6 +173,7 @@ impl Context {
             return_params: return_params.clone(),
             email: email.to_owned(),
             email_addr: email_addr.clone(),
+            response_type,
             nonce: nonce.to_owned(),
             signing_alg,
         });
@@ -192,7 +227,7 @@ pub type Response = hyper::Response<Body>;
 /// Result type of handlers
 pub type HandlerResult = Result<Response, BrokerError>;
 
-// HTTP service
+/// HTTP service
 pub struct Service {
     /// The application configuration
     app: ConfigRc,
@@ -209,6 +244,8 @@ impl Service {
     }
 
     async fn serve(ip: IpAddr, req: Request, app: ConfigRc) -> Result<Response, BoxError> {
+        metrics::HTTP_REQUESTS.inc();
+
         // Handle only simple path requests.
         if req.uri().scheme_str().is_some() || req.uri().host().is_some() {
             let mut response = empty_response(StatusCode::BAD_REQUEST);
@@ -217,9 +254,9 @@ impl Service {
         }
 
         // Read the request body.
-        let (parts, body) = req.into_parts();
+        let (parts, mut body) = req.into_parts();
         let body = match parts.method {
-            Method::POST => read_body(body).await?,
+            Method::POST => read_body(&mut body).await?,
             _ => Bytes::from(vec![]),
         };
 
@@ -231,7 +268,7 @@ impl Service {
             .and_then(|value| value.to_str().ok())
         {
             'lang: for user_language in &accept_language::parse(user_languages) {
-                for (idx, &(ref lang, _)) in app.i18n.catalogs.iter().enumerate() {
+                for (idx, &(lang, _)) in app.i18n.catalogs.iter().enumerate() {
                     if lang == user_language {
                         catalog_idx = idx;
                         break 'lang;
@@ -241,6 +278,10 @@ impl Service {
         }
 
         // Create the request context.
+        let want_json = parts
+            .headers
+            .get(hyper::header::ACCEPT)
+            .map_or(false, |accept| accept == "application/json");
         let mut ctx = Context {
             app,
             ip,
@@ -252,6 +293,7 @@ impl Service {
             session_data: None,
             catalog_idx,
             return_params: None,
+            want_json,
         };
 
         // Call the route handler.
@@ -266,6 +308,16 @@ impl Service {
         // Set common response headers.
         set_headers(&mut response);
 
+        // Response status metrics.
+        match response.status().as_u16() {
+            100..=199 => metrics::HTTP_RESPONSE_STATUS_1XX.inc(),
+            200..=299 => metrics::HTTP_RESPONSE_STATUS_2XX.inc(),
+            300..=399 => metrics::HTTP_RESPONSE_STATUS_3XX.inc(),
+            400..=499 => metrics::HTTP_RESPONSE_STATUS_4XX.inc(),
+            500..=599 => metrics::HTTP_RESPONSE_STATUS_5XX.inc(),
+            _ => {}
+        }
+
         Ok(response)
     }
 }
@@ -310,15 +362,12 @@ impl HyperService<Request> for Service {
 async fn handle_error(ctx: &Context, err: BrokerError) -> Response {
     let reference = err.log(Some(&ctx.app.rng)).await;
 
-    if ctx.want_json() {
-        let mut res = json_response(
-            &json!({
-                "error": err.oauth_error_code(),
-                "error_description": &format!("{}", err),
-                "reference": reference,
-            }),
-            None,
-        );
+    if ctx.want_json {
+        let mut res = json_response(&json!({
+            "error": err.oauth_error_code(),
+            "error_description": &format!("{}", err),
+            "reference": reference,
+        }));
         *res.status_mut() = err.http_status_code();
         return res;
     }
@@ -443,7 +492,7 @@ pub fn parse_form_encoded(input: &[u8]) -> HashMap<String, String> {
 }
 
 /// Read the request or response body up to a fixed size.
-pub async fn read_body(mut body: Body) -> Result<Bytes, BoxError> {
+pub async fn read_body(body: &mut Body) -> Result<Bytes, BoxError> {
     let mut acc = BytesMut::new();
     while let Some(result) = body.next().await {
         let chunk = result.map_err(Box::new)?;
@@ -501,6 +550,15 @@ pub fn return_to_relier(ctx: &Context, params: &[(&str, &str)]) -> Response {
 
             html_response(ctx.app.templates.forward.render_data(&data))
         }
+        // Add params as query parameters and redirect.
+        ResponseMode::Query => {
+            let mut redirect_uri = redirect_uri.clone();
+            redirect_uri.query_pairs_mut().extend_pairs(params).finish();
+
+            let mut res = empty_response(StatusCode::SEE_OTHER);
+            res.header(hyper::header::LOCATION, String::from(redirect_uri));
+            res
+        }
     }
 }
 
@@ -508,13 +566,10 @@ pub fn return_to_relier(ctx: &Context, params: &[(&str, &str)]) -> Response {
 ///
 /// Serializes the argument value to JSON and returns a HTTP 200 response
 /// code with the serialized JSON as the body.
-pub fn json_response(obj: &serde_json::Value, max_age: Option<Duration>) -> Response {
-    let body = serde_json::to_string(&obj).expect("unable to coerce JSON Value into string");
+pub fn json_response(obj: &serde_json::Value) -> Response {
+    let body = serde_json::to_string_pretty(&obj).expect("unable to coerce JSON Value into string");
     let mut res = Response::new(Body::from(body));
     res.typed_header(ContentType::json());
-    if let Some(max_age) = max_age {
-        res.typed_header(CacheControl::new().with_public().with_max_age(max_age));
-    }
     res
 }
 
